// Project: Safe terminal
// Homepage: www.HomoFaciens.de
// Author Norbert Heinz
// Version: 0.1
// Creation date: 06.03.2022
// compile with:
// gcc USB-transfer.c -o USB-transfer -lm
// Eventually you must install libc6-dev with:
// sudo apt-get install libc6-dev
// User must be member of group "dialout" (eventually reboot to take effect):
// usermod -a -G dialout <your username>
// Details on:
// http://www.HomoFaciens.de/technics-physical-computing-Secure-Communication-Terminal_ge.htm

#include <stdio.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>


#define BAUDRATE B115200
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"
#define FALSE 0
#define TRUE 1


FILE *FileOut, *FileIn;
char fileName[30000];
char decryptFileName[30000];
char OpenDir[1000];
char DoneDir[1000];
char command[100000];


int fd = 0;
struct termios TermOpt;


//+++++++++++++++++++++++ Start readport ++++++++++++++++++++++++++
unsigned char  readport(void){
  int n;
    char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//------------------------ End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport ++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend, int waitForReady){
  int n;

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\r");
  }
  else{
    if(waitForReady != 0){
      while(readport() != 'r');
    }
  }

}
//------------------------ End sendport ----------------------------------

//+++++++++++++++++++++++ Start openport ++++++++++++++++++++++++++
void openport(void){

    fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)  {
      printf("init_serialport: Unable to open port ");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      printf("init_serialport: Couldn't get term attributes");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      printf("init_serialport: Couldn't set term attributes");
    }

}
//------------------------ End openport ----------------------------------

//+++++++++++++++++++++++ Start messageWatchdog ++++++++++++++++++++++++
void messageWatchdog(){
  DIR *pDIR;
  struct dirent *pDirEnt;
  int byteCounter = 0;
  unsigned char a;

  pDIR = opendir(OpenDir);
  if (pDIR == NULL){
    printf("Could not open directory '%s'!", OpenDir);
    exit(1);
  }
  pDirEnt = readdir(pDIR);
  while (pDirEnt != NULL){
    if(strlen(pDirEnt->d_name) > 4){
      if(memcmp(pDirEnt->d_name + strlen(pDirEnt->d_name)-4, ".zip",4) == 0){
        sleep(1); // give operating system one second to finish the file copy
        strcpy(decryptFileName, OpenDir);
        strcat(decryptFileName, pDirEnt->d_name);
        printf("Sending file %s ...\n", decryptFileName);
        if((FileIn=fopen(decryptFileName,"rb"))==NULL){
          printf("Can't open file '%s'!\n", decryptFileName);
          exit(1);
        }
        byteCounter = 0;
        while(!(feof(FileIn))){
          if(fread(&a, 1, 1, FileIn)==1){
            //printf("a=%c=%d,", a, a);
            sendport(a, 1);
            byteCounter++;
          }
        }
        fclose(FileIn);
        sprintf(command, "mv %s %s", decryptFileName, DoneDir);
        //sleep(1); // give operating system one second to finish the file delete
        printf("Moving: %s", command);
        system(command);
        printf("...%d Bytes done\n", byteCounter);
      }
    }
    pDirEnt = readdir(pDIR);
  }
  closedir(pDIR);
}
//------------------------ End messageWatchdog -------------------------



//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  unsigned char a;
  unsigned long lastRead = 0;
  int readMode = 0;
  int byteCounter = 0;

  openport();

//  printf("\n\nWaiting for 'X' from Arduino (Arduino pluged in?)...\n");

  //Wait for 'X' from Arduino
//  while(readport() != 'X');
//  sendport('r', 0);
//  printf("\n\n...connection to Arduino established!\n");

  //strcpy(FileName, "noFiLE");

  getcwd(OpenDir, 1000);
  strcat(OpenDir, "/decrypt/");
  strcpy(DoneDir, OpenDir);
  strcat(DoneDir, "done/");
  printf("OpenDir=>%s<\n", OpenDir);

  printf("Empty buffer...");
  sleep(1);
  a = readport();
  while(a != 0){
    printf("%c", a);
    sleep(1);
    a = readport();
  }
  printf("...nothing in buffer!\n\n");

  while (1){

    messageWatchdog();

    a = readport();
    printf("%c", a);
    if(a != 0){
      byteCounter++;
      lastRead = time(NULL);
      if(readMode == 0){
        readMode = 1;
        printf("Receiving message:");
        sprintf(fileName, "%ld_encrypt.zip", time(NULL));
        if((FileOut=fopen(fileName,"wb"))==NULL){
          printf("Can't open file '%s'!\n", fileName);
          return(1);
        }
      }
      printf("%c", a);
      fwrite(&a, 1, 1, FileOut);
      sendport('r', 0);
    }
    else{
      // Nothing to do
      // Sleep a while to reduce CPU load
      usleep(500000);
    }

    if(readMode == 1 && time(NULL) - lastRead > 3){
      readMode = 0;
      fclose(FileOut);
      printf("\n\n%d Bytes of encrypted message stored in %s\n", byteCounter, fileName);
      byteCounter = 0;
    }

  }

  return 0;
}
